;;;
;;;    LYDELMRG.LSP
;;;
;;;    By Dominic Panholzer
;;;
;;;    Copyright  1999 by Autodesk, Inc.
;;;
;;;    Your use of this software is governed by the terms and conditions of the
;;;    License Agreement you accepted prior to installation of this software.
;;;    Please note that pursuant to the License Agreement for this software,
;;;    "[c]opying of this computer program or its documentation except as
;;;    permitted by this License is copyright infringement under the laws of
;;;    your country.  If you copy this computer program without permission of
;;;    Autodesk, you are violating the law."
;;;
;;;    AUTODESK PROVIDES THIS PROGRAM "AS IS" AND WITH ALL FAULTS.
;;;    AUTODESK SPECIFICALLY DISCLAIMS ANY IMPLIED WARRANTY OF
;;;    MERCHANTABILITY OR FITNESS FOR A PARTICULAR USE.  AUTODESK, INC.
;;;    DOES NOT WARRANT THAT THE OPERATION OF THE PROGRAM WILL BE
;;;    UNINTERRUPTED OR ERROR FREE.
;;;
;;;    Use, duplication, or disclosure by the U.S. Government is subject to
;;;    restrictions set forth in FAR 52.227-19 (Commercial Computer
;;;    Software - Restricted Rights) and DFAR 252.227-7013(c)(1)(ii)
;;;    (Rights in Technical Data and Computer Software), as applicable.
;;;
;;;  ----------------------------------------------------------------
;;;
;;;  External Functions:
;;;
;;;     ACET-ERROR-INIT       --> ACETUTIL.FAS   Intializes bonus error routine
;;;     ACET-ERROR-RESTORE    --> ACETUTIL.FAS   restores old error routine
;;;     ACET-LAYER-LOCKED     --> ACETUTIL.FAS   Checks to see if layer is locked
;;;     ACET-TABLE-NAME-LIST  --> ACETUTIL.FAS   Returns a list of table enties
;;;     ACET-SPINNER          --> ACETUTIL.FAS   Shows spinner while work is being done

; -------------------- LAYER DELETE FUNCTION ---------------------
; Deletes layer from drawing.
; ----------------------------------------------------------------

(defun C:LAYDEL ()
  (if (= (getvar "fullopen") 1)
    (acet-lydelmrg-main "delete")
    (princ "\nLAYLSCH kann nicht auf partiell geffnete Zeichnungen angewendet werden.")
  )
  (princ)
)

; --------------------- LAYER MERGE FUNCTION ---------------------
; Merges layer with second layer.
; ----------------------------------------------------------------

(defun C:LAYMRG ()
  (if (= (getvar "fullopen") 1)
    (acet-lydelmrg-main "merge")
    (princ "\nLAYMISCH kann nicht auf partiell geffnete Zeichnungen angewendet werden.")
  )
  (princ)
)

; ------------- LAYER PROCESSOR FOR LAYDEL & LAYMRG --------------
;  Main program body for LAYDEL and LAYMRG.
;  This function will remove all the entities on a selected layer and
;  remove the layer from the drawing. If the merge option is
;  selected, all the entities on the layer to be removed will be
;  changed to the merging layer. Block definitions referencing the
;  layer to be removed will be redefined with the dependencies
;  removed (or changed when merging.)
; ----------------------------------------------------------------

(defun acet-lydelmrg-main (MODE / acet-lydelmrg-blkredef acet-lydelmrg-attredef acet-lydelmrg-layinblklst
                                  acet-lydelmrg-layoutlist acet-lydelmrg-vpcheck
                                  ANS LOOP LAYLST CLAY LAYSTR LAY VAL LST BLKLST STR FLTR
                                  SS CNT EN PASS NEWLAY CVPORT NOPURGE_LST ALERT_STRING 
                                  LAYOUTLST CLAYOUT)

; --------------------- BLKREDEF FUNCTION ------------------------
;   This function redefines block BNAM removing dependencies on
;   layers in the list LAYLST. MODE can be either
;   "delete" or "merge". If merge is used NEWLAY specifies
;   the target layer.
; ----------------------------------------------------------------

  (defun acet-lydelmrg-blkredef (BNAM LAYLST MODE NEWLAY / BLK EN ED ILAY ABLK FLTR)

    (entmake)

    (setq BLK (entget (tblobjname "BLOCK" BNAM)'("*"))    ; Get first block definition
          BLK (subst (cons 8 "0") (assoc 8 BLK) BLK)      ; Make sure the def is on lay 0
          EN  (cdr (assoc -2 BLK))                        ; and the first subentity
    )

    (entmake BLK)                                         ; Start redefining the block
    (while EN                                             ; While there are subentities

      (setq ED (entget EN '("*")))
      (if (or
            (= "POLYLINE" (cdr (assoc 0 ED)))             ; If the subent is a polyline
            (and
              (= "INSERT" (cdr (assoc 0 ED)))             ; or if the subent is a block
              (= 1 (cdr (assoc 66 ED)))                   ; with attributes
            )
          )
        (setq ABLK T                                      ; then set the complex entity flag
              ILAY (cdr (assoc 8 ED))                     ; and save the layer.
        )
      )

      (if (member (cdr (assoc 8 ED)) LAYLST)              ; If its layer is in the list
        (progn
          (if (= (strcase MODE) "MERGE")                  ; if the layer is to be merged
            (progn
              (setq ED (subst (cons 8 NEWLAY) (assoc 8 ED) ED))  ; substitute the old layer with new
              (entmake ED)                                ; and add the entity to the definition
            )
            (progn                                        ; else if layer is to be deleted
              (if (= "SEQEND" (cdr (assoc 0 ED)))         ; If the ent is a seqend
                (progn                                    ; change its layer to that of the insert
                  (setq ED (subst (cons 8 ILAY) (assoc 8 ED) ED))
                  (entmake ED)                            ; and add it to the definition.
                )
              )
              (if ABLK                                    ; If the complex entity flag is set,
                (while (/= "SEQEND" (cdr (assoc 0 ED)))   ; while the entity is not a seqend,
                  (setq EN (entnext EN)                   ; step through the sub entities
                        ED (entget EN '("*"))             ; removing them from the definition.
                  )
                )
              )
            )
          )
        )
        (entmake ED)                                      ; If layer does not match, add
      )                                                   ; the entity to the definition
      (setq EN   (entnext EN)                             ; get the next subentity
            ABLK nil                                      ; and reset the attribute block flag.
      )
    )

    (setq BNAM (entmake (list (cons 0 "ENDBLK"))))        ; complete the redefinition

    (if (and (= (strcase MODE) "DELETE")                  ; If in delete mode and
             (not (entnext (tblobjname "BLOCK" BNAM)))    ; the block definition has no entities
        )
      (progn
        (Setq FLTR (list '(-4 . "<AND")
                           '(0 . "INSERT")
                           (cons 2 BNAM)
                         '(-4 . "AND>")
                   )
        )
        (if (setq SS  (ssget "_x" FLTR))
          (command "_.erase" SS "")                       ; erase all the insertions
        )
        (if (setq SS  (ssget "_x" FLTR))
          (progn
            (setq CNT 0)
            (while (and ss
                        (> (sslength ss) 0)
                        (setq EN (ssname SS CNT))
                   );and
              (entdel EN)
              (setq CNT (1+ CNT))
            )
          )
        )
        (if (tblobjname "BLOCK" BNAM)
          (command "_.purge" "_block" BNAM "_n")         ; and purge the block
        )
      )
    )
  )

; ----------------- ATTRIBUTE REDEF FUNCTION ---------------------
;   This function recreates block insertions which have
;   attributes which reference layer LAY. MODE can be either
;   "delete" or "merge". If merge is used NEWLAY specifies
;   the target layer.
; ----------------------------------------------------------------

  (defun acet-lydelmrg-attredef (LAY MODE NEWLAY / FLTR SS CNT EN ILAY ELST DOIT LAYLCK)
    (setq FLTR (list '(0 . "INSERT") '(66 . 1))
          SS   (ssget "_x" FLTR)
          CNT  0
    )
    (while (and ss
                (> (sslength ss) 0)
                (setq EN (ssname SS CNT))
           );and
      (setq ED   (entget EN '("*"))
            ILAY (cdr (assoc 8 ED))
            ELST (list ED)
            DOIT nil
      )
      (while (/= "SEQEND" (cdr (assoc 0 (entget (setq EN (entnext EN))))))
        (setq ED (entget EN '("*")))
        (if (/= LAY (cdr (assoc 8 ED)))
          (setq ELST (cons ED ELST))
          (progn
            (if (= (strcase MODE) "MERGE")
              (setq ED   (subst (cons 8 NEWLAY) (assoc 8 ED) ED)
                    ELST (cons ED ELST)
              )
            )
            (setq DOIT T)
          )
        )
      )

      (setq ELST (reverse ELST)
            ED   (entget EN '("*"))
      )

      (if (< 1 (length ELST))
        (if (or DOIT (= LAY (cdr (assoc 8 ED))))
          (progn
            ;;(entmake nil)
            (entmake)
            (foreach MBR ELST (entmake MBR))
            (entmake (subst (cons 8 ILAY) (assoc 8 ED) ED))
            (entdel (cdr (assoc -1 (car ELST))))
          )
        )
        (progn
          ;;(entmake nil)
          (entmake)
          (entmake (subst (cons 66 0) (assoc 66 (car ELST)) (car ELST)))
          (if (setq LAYLCK (acet-layer-locked (cdr (assoc 8 (car ELST)))))
            (command "_.-layer" "_u" (cdr (assoc 8 (car ELST))) "")
          )
          (entdel (cdr (assoc -1 (car ELST))))
          (if LAYLCK (command "_.-layer" "_lo" (cdr (assoc 8 (car ELST))) ""))
        )
      )
      (setq CNT (1+ CNT))
    )
  )

; --------------- GET REFERENCED LAYERS FUNCTION -----------------
;  Returns list of blocks which reference Layer LAY
; ----------------------------------------------------------------

  (defun acet-lydelmrg-layinblklst (LAY / BLK NAM EN NAMLST)

    (princ "\nBlocktabellenabfrage  ")

    (setq BLK    (tblnext "BLOCK" T))                     ; Get first block definition
    (while BLK                                            ; If there is a block
      (if (= (logand (cdr (assoc 70 BLK)) 32) 0)          ; and it is not xref dependent,
        (progn
          (setq NAM    (cdr (assoc 2 BLK))                ; get the name
                EN     (tblobjname "BLOCK" NAM)           ; and the first entity
          )
          (while EN                                       ; while there is a subentity
            (if (= LAY (cdr (assoc 8 (entget EN))))       ; if the ent is on the layer
              (setq NAMLST (if NAMLST                     ; add the name to the list
                             (append NAMLST (list NAM))
                             (list NAM)
                           )
                    EN     nil                            ; and skip other subentities
              )
              (setq EN (entnext EN))                      ; else get the next subentity
            )
            (acet-spinner)
          )
        )
      )
      (setq BLK (tblnext "BLOCK"))                        ; get the next block
    )
    (prompt "\r                      ")
    (if (/= NAMLST "")
      NAMLST                                              ; return the list of blocks
    )
  )


; -------------------- LAYOUT LIST FUNCTION ----------------------
;   Returns list of layouts that have viewports that reference
;   the layers in LAYLST via vplayer freeze or by being on the layer.
;   If MODE == "Merge" the layout list will include those layouts
;   with any entities on the layer to be merged.
; ----------------------------------------------------------------

  (defun acet-lydelmrg-layoutlist (LAYLST MODE / SS LAYOUTS CNT VP XLST LAYOUTNAME CLAYOUT
                                             LAYOUTLST ADD VPFRZ DEPLAYLST LAYFLTR LAYOUTFLTR
                                   )
  
    (setq LAYOUTS (cons "Model" (layoutlist))            ; Get a list of all the layouts
          LAYOUTS (vl-remove (getvar "ctab") LAYOUTS)    ; except the current one
          LAYOUTFLTR (mapcar '(lambda (X) (cons 410 X)) LAYOUTS)      ; Make a layout filter list
          LAYOUTFLTR (append '((-4 . "<OR")) LAYOUTFLTR '((-4 . "OR>")))
          CNT   0
    )

    (setq SS (ssget "_x" (append '((-4 . "<AND"))        ; Get all the viewports in all layouts
                                   '((0 . "VIEWPORT"))   ; except the current one.
                                   LAYOUTFLTR
                                 '((-4 . "AND>"))
                         )
             )
    )

    (if (and SS (> (sslength SS) 0))                     ; if there are any
      (progn 
        (while (and
                 LAYOUTS                                 ; while not all layouts have been used
                 (setq VP (ssname SS CNT))               ; step through each viewport
               )

          (setq VP         (entget VP '("ACAD"))         ; get the viewport's edata
                LAYOUTNAME (cdr (assoc 410 VP))          ; get the viewport's layout
                XLST       (cdadr (assoc -3 VP))         ; get the xdata list
          )

          (if (member (cdr (assoc 8 VP)) LAYLST)         ; if the vp is on the layer
            (setq ADD   T                                ; earmark the layout to be added
                  VPFRZ T                                ; and increment the vpfrz counter
            )
          )

          (if (not ADD)                                  ; If the layout has not been earmarked
            (progn
              (setq DEPLAYLST (acet-list-m-assoc 1003 XLST) ; Find all the dependant layers
                    DEPLAYLST (mapcar 'cdr DEPLAYLST)       ; and get their names
              )
              (while DEPLAYLST                       ; while the dependant name list exists,
                (if (member (car DEPLAYLST) LAYLST)  ; if the first member is in LAYLST
                  (setq ADD       T
                        DEPLAYLST nil                ; Stop processing this viewport
                  )
                  (setq DEPLAYLST (cdr DEPLAYLST))   ; remove the first item of the dependant list
                )
              )
            )
          )
          (if (and ADD (not (member LAYOUTNAME LAYOUTLST))) ; If the layout has not already been added
            (setq LAYOUTLST (cons LAYOUTNAME LAYOUTLST)     ; Add it to the list,
                  LAYOUTS   (vl-remove LAYOUTNAME LAYOUTS)  ; remove it from the master list,
                  ADD       nil                             ; and reset the ADD flag
            )
            (setq ADD nil)                                  ; otherwise just reset the ADD flag
          )
          (setq CNT (1+ CNT))
        )
      )
    )

    (if (= (strcase MODE) "DELETE")
      (setq CLAYOUT (vla-get-activedocument (vlax-get-acad-object))
            CLAYOUT (vla-get-name (vla-get-layout (vla-get-paperspace CLAYOUT)))
            LAYOUTS (vl-remove CLAYOUT LAYOUTS)
      )
    )

    (if LAYOUTS
      (progn
        (setq LAYFLTR (mapcar '(lambda (X) (cons 8 X)) LAYLST)         ; Make a layer filter list
              LAYFLTR (append '((-4 . "<OR")) LAYFLTR '((-4 . "OR>")))
        )
        (foreach LAYOUT LAYOUTS
          (if (ssget "_x" (append
                            '((-4 . "<AND"))
                              (list (cons 410 LAYOUT))
                              LAYFLTR
                            '((-4 . "AND>"))
                          )
              )
            (setq LAYOUTLST (cons LAYOUT LAYOUTLST))                   ; Add the layout to the list
          )
        )
      )
    )
    (if LAYOUTLST (setq LAYOUTLST (cons VPFRZ LAYOUTLST)))
  )


; ----------------- VIEWPORT CHECKING FUNCTION -------------------
;   This function will check for viewports being referenced by
;   any layer in LAYLST. If it finds any it will clear the
;   references via VPLAYER. If tilemode is 1 and referenced
;   viewports exist, The user is asked permission to switch tilemode.
;   Returns T if no viewports exist.
;   Returns nil if vp exist but user does not want to switch tilemode.
;   Else returns active viewport number.
; ----------------------------------------------------------------

  (defun acet-lydelmrg-vpcheck (LAYLST MODE / LAYOUTLST VPON PASS CLAYOUT CVPORT)

    (setq LAYOUTLST (acet-lydelmrg-layoutlist LAYLST MODE))

    (if LAYOUTLST                                        ; If layers are being referenced
      (if (= (logand (getvar "cmdactive") 4) 4)          ; if a script is active
        (setq PASS "Yes")                                ; skip the questions
        (progn
          (setq VPON      (car LAYOUTLST)                ; retrieve the VP on layer flag
                LAYOUTLST (cdr LAYOUTLST)
          )
          (cond
            ((< 1 (length LAYOUTLST))                    ; More than one layout with vpfreeze referenced layer(s)
              (terpri)
              (prompt (acet-str-format "\nEs sind %1 andere Layouts vorhanden, die einen oder mehr der ausgewhlten Layer referenzieren."  (length LAYOUTLST)))
              (prompt "\nDamit dieser Befehl fortgefhrt werden kann, mssen diese Layouts zu aktuellen Layouts gemacht werden.")
            )
            ((and
               (/= (getvar "ctab") (car LAYOUTLST))      ; One layout that is not the current layout
               (= 1 (length LAYOUTLST))                  ; with vpfreeze referenced layer(s)
             )
              (terpri)
              (prompt "\nEs ist ein weiteres Layout vorhanden, das einen oder mehr der ausgewhlten Layer referenziert.")
              (prompt "\nDamit dieser Befehl fortgefhrt werden kann, mu dieses Layout zum aktuellen Layout gemacht werden." )
            )
          )

          (if (and VPON (= (strcase MODE) "DELETE"))
            (progn
              (terpri)
              (prompt "\nEs sind ein oder mehr Papierbereichsansichtsfenster vorhanden, die sich auf einem der")
              (prompt "\nausgewhlten Layer befinden. DIESE ANSICHTSFENSTER WERDEN GELSCHT.")
            )
          )

          (textscr)
          (terpri)
          (initget "Ja Nein _Yes No")

          (setq PASS (getkword "\nMchten Sie fortfahren? [Ja/Nein] <Nein>:")
                PASS (if PASS PASS "No")
          )

        )
      )
    )
    
    (cond
      ((= PASS "Yes")                                                    ; If layout switching is needed
        (setq LAYOUTLST (cons (getvar "ctab") LAYOUTLST))                ; add the current layout to the list
      )
      ((= PASS "No")                                                     ; If cancel is desired
        (setq LAYOUTLST nil)                                             ; return nil
      )
      (T
        (setq LAYOUTLST (list (getvar "ctab")))                          ; otherwise just return the current layout
      )
    )
    LAYOUTLST
  )

; ----------------------------------------------------------------
;                          MAIN PROGRAM
; ----------------------------------------------------------------

  (acet-error-init
    (list
      (list "cmdecho" 0
            "cecolor" "bylayer"
            "pickstyle" 0

      )
      T     ;flag. True means use undo for error clean up.
      '(if (and (= (strcase MODE) "MERGE") LAYLST)
         (foreach LAY LAYLST
           (setq SS  (ssget "_x" (list (cons 8 LAY)))
                 CNT 0
           )
           (while (and ss
                       (> (sslength ss) 0)
                       (setq EN (ssname SS CNT))
                  );and
             (redraw EN 4)
             (setq CNT (1+ CNT))
           )
         )
       )
    );list
  );acet-error-init


  (setq ANS  "START"
        LOOP T
        CLAY (getvar "clayer")
  )

  (command "_.undo" "_m")
  (while LOOP
    (cond
      ((not ANS)
        (setq LOOP nil)
      )
      ((listp ANS)
        (setq LAY (cdr (assoc 8 (entget (car ANS)))))
      )
      ((= ANS "Type-it")
        (setq LAY (getstring "\nLayer-Namen eingeben oder [?]: " T))
        (cond
          ((= LAY "")
            (setq ANS nil
                  LAY nil
            )
          )
          ((= LAY "?")
            (setq STR (getstring "\nAufzulistende(n) Layer-Namen eingeben <*>:"))
            (textscr)
            (setvar "qaflags" 2)
            (command "_.layer" "?" STR)
            (while (< 0 (getvar "cmdactive")) (command ""))
            (setvar "qaflags" 0)
            (setq LAY nil)
          )
          ((setq LAY (tblobjname "LAYER" LAY))
            (setq LAY (cdr (assoc 2 (entget LAY)))     ; get its real name
                  ANS T
            )
          )
          (T
            (prompt "\nUngltiger Layer-Name.")
            (setq LAY nil)
          )
        )
      )
      ((= ANS "Undo")
        (if LAYLST
          (progn
            (if (= (strcase MODE) "DELETE")
              (command "_.undo" "1")
            )
            (if (= (strcase MODE) "MERGE")
              (progn
                (setq SS  (ssget "_x" (list (cons 8 (car LAYLST))))
                      CNT 0
                )
                (while (and ss
                            (> (sslength ss) 0)
                            (setq EN (ssname SS CNT))
                       );and
                  (redraw EN 4)
                  (setq CNT (1+ CNT))
                )
              )
            )
            (setq LAYLST (cdr LAYLST))
          )
          (prompt "\nAlle Aktionen wurden rckgngig gemacht.")
        )
      )
    )

    (if LAY
      (cond
        ((= LAY "0")
          (if (= (strcase MODE) "DELETE")
            (prompt "\nKann Layer 0 nicht lschen.")
            (prompt "\nKann Layer 0 nicht mischen.")
          )
        )
        ((= LAY "DEFPOINTS")
          (if (= (strcase MODE) "DELETE")
            (prompt "\nKann Layer DEFPOINTS nicht lschen.")
            (prompt "\nKann Layer DEFPOINTS nicht mischen.")
          )
        )
        ((wcmatch LAY "*|*")
          (if (= (strcase MODE) "DELETE")
            (prompt (acet-str-format "\nLayer %1 ist extern abhngig und kann nicht gelscht werden." LAY))
            (prompt (acet-str-format "\nLayer %1 ist extern abhngig und kann nicht gemischt werden." LAY))
          ) ;if
        )
        ((and
           (= LAY CLAY)
           (= (strcase MODE) "DELETE")
         )
          (prompt "\nKann aktuellen Layer nicht lschen.")
        )
        ((acet-layer-locked LAY)
          (if (= (strcase MODE) "DELETE")
            (prompt (acet-str-format "\nLayer %1 ist gesperrt und kann nicht gelscht werden." LAY))
            (prompt (acet-str-format "\nLayer %1 ist gesperrt und kann nicht gemischt werden." LAY))
          ) ;if
        )
        ((not (member LAY LAYLST))
          (setq LAYLST (if LAYLST
                         (append (list LAY) LAYLST)
                         (list LAY)
                       )
          )
          (if (= (strcase MODE) "DELETE")
            (command "_.-layer" "_f" LAY "")
          )
          (if (= (strcase MODE) "MERGE")
            (progn
              (setq SS  (ssget "_x" (list (cons 8 LAY)))
                    CNT 0
              )
              (while (and ss
                          (> (sslength ss) 0)
                          (setq EN (ssname SS CNT))
                     );and
                (redraw EN 3)
                (setq CNT (1+ CNT))
              )
            )
          )
        )
      )
    )

    (if (and LOOP (/= ANS "Type-it"))
      (progn
        (setvar "errno" 7)
        (setq LAY nil)
        (while (= (getvar "errno") 7)
          (setvar "errno" 0)
          (initget "Eingeben Zurck _Type-it Undo ")
          (if LAYLST
            (progn
              (setq LAYSTR "")
              (foreach VAL LAYLST (setq LAYSTR (strcat LAYSTR VAL ",")))
              (setq LAYSTR (substr LAYSTR 1 (1- (strlen LAYSTR))))
              (prompt (acet-str-format "\nAusgewhlte Layer: %1" LAYSTR))
              (if (= (strcase MODE) "DELETE")
                (setq ANS  (entsel "\nZu lschendes Objekt auf Layer auswhlen oder [Eingeben/Zurck] <fertig>: "))
                (setq ANS  (entsel "\nZu mischendes Objekt auf Layer auswhlen oder [Eingeben/Zurck] <fertig>: "))
              )
            )
            (if (= (strcase MODE) "DELETE")
              (setq ANS  (entsel "\nZu lschendes Objekt auf Layer auswhlen oder [Eingeben/Zurck]: "))
              (setq ANS  (entsel "\nZu mischendes Objekt auf Layer auswhlen oder [Eingeben/Zurck]: "))
            )
          )
          (if (= (getvar "errno") 7)
            (prompt "\nNichts ausgewhlt.")
          )
        )
      )
    )
  ); while LOOP


  (if LAYLST
    (progn

; ----------------------- build BLKLST ---------------------------
; BLKLST has the form: (("blk1" "lay1") ("blk2" "lay1" "lay2")...)
; ----------------------------------------------------------------

      (foreach LAY LAYLST                                ; Check each layer selected
        (setq LST (acet-lydelmrg-layinblklst LAY))       ; if it is referenced in a block
        (if (and LST (not BLKLST))                       ; if so and no BLKLST
          (setq BLKLST                                   ; build BLK LST
            (mapcar '(lambda (x)
                      (list x LAY)
                    )
                    LST
            )
          )
          (foreach BLK LST                               ; else go through each referenced block
            (if (setq VAL (assoc BLK BLKLST))                           ; if the block already is in BLKLST
              (setq BLKLST (subst (append VAL (list LAY)) VAL BLKLST))  ; update the entry to include layer
              (setq BLKLST (append BLKLST (list (list BLK LAY))))       ; else make a new entry
            )
          )
        )
      )

      (cond

; ------------------------ Layer Delete --------------------------

        ((= (strcase MODE) "DELETE")
          (prompt "\n******** WARNUNG ********")
          (if BLKLST
            (progn
              (textscr)
              (terpri)
              (terpri)
              (prompt (acet-str-format "\nEs sind %1 Blockdefinition(en) vorhanden, die den (die) zu lschenden Layer referenzieren."
                        (rtos (length BLKLST) 2 0)
                      ))
              (prompt (strcat "\nDer oder die Blcke werden neu definiert, und die Elemente, die den (die) Layer\nreferenzieren, werden aus der (den) Blockdefinition(en) entfernt."))
            )
          )
          (terpri)
          (if (= (length LAYLST) 1)
            (prompt (acet-str-format "\nSie sind dabei, Layer %1 endgltig aus dieser Zeichnung zu lschen."  (car LAYLST)))
            (progn
              (textscr)
              (prompt "\nSie sind dabei, die folgenden Layer endgltig aus dieser Zeichnung zu lschen:\n")
              (foreach LAY LAYLST
                (prompt (strcat "\n" LAY))
              )
              (terpri)
            )
          )
          (initget "Ja Nein _Yes No")
          (setq PASS (getkword "\nMchten Sie fortfahren? [Ja/Nein] <Nein>:"))
          (setq PASS (if (null PASS) "No" PASS)) ;setq

          (if (and
                (= PASS "Yes")
                (setq LAYOUTLST (acet-lydelmrg-vpcheck LAYLST MODE))
              )

            (progn
              (setq FLTR (mapcar '(lambda (X) (cons 8 X)) LAYLST)         ; Make a layer filter list
                    FLTR (append '((-4 . "<OR")) FLTR '((-4 . "OR>")))
              )

              (if BLKLST                               ; Clean up block table
                (foreach BLK BLKLST                    ; by going through each block in BLKLST
                  (acet-lydelmrg-blkredef (car BLK) (cdr BLK) MODE NEWLAY) ; and removing the referencing entities
                )
              )


              (progn
                (setq CLAYOUT (getvar "ctab")
                      CVPORT (getvar "cvport")
                )
                (foreach LAYOUT LAYOUTLST
                  (setvar "ctab" LAYOUT)
                  (if (/= LAYOUT "Model")
                    (progn
                      (command "_.pspace")
                      (command "_.vplayer" "_t" LAYSTR "_a" "" )
                    )
                  )
                  (if (setq SS  (ssget "_x" FLTR))
                    (command "_.erase" SS "")
                  )
                )
                (setvar "ctab" CLAYOUT)

                (if (and (/= CLAYOUT "Model")
                         (< 1 CVPORT)
                         (ssget "_x" '((0 . "VIEWPORT")))
                    )
                  (command "_.mspace")
                )
              )

              (if (setq SS  (ssget "_x" FLTR))
                (progn
                  (setq CNT 0)
                  (while (and ss
                              (> (sslength ss) 0)
                              (setq EN (ssname SS CNT))
                         );and
                    (entdel EN)
                    (setq CNT (1+ CNT))
                  )
                )
              )

              (foreach LAY LAYLST
                (acet-lydelmrg-attredef LAY MODE NEWLAY)             ; Recreate inserts which reference LAY
                (terpri)
              )

              (if (vl-remove nil (mapcar '(lambda (x) (wcmatch x "`*D*")) (acet-table-name-list '("BLOCK"))))
                (command "_.purge" "_block" "`*D*" "_n")            ; Purge all anonymous dim blocks
              )

              (command "_.purge" "_layer" LAYSTR "_n")              ; Purge all the layers

              (foreach LAY LAYLST
                (if (tblobjname "LAYER" LAY)
                  (setq NOPURGE_LST (append NOPURGE_LST (list LAY)))
                )
              )

              (if NOPURGE_LST
                (progn
                  (if (< 1 (length NOPURGE_LST))
                    (progn
                      (setq LAYSTR "\n")
                      (foreach VAL NOPURGE_LST (setq LAYSTR (strcat LAYSTR VAL "\n")))
                      (setq ALERT_STRING
                        (acet-str-format "Die folgenden Layer konnten nicht bereinigt werden, da sie\ndurch (ein) Objekt(e) in der Zeichnung referenziert werden:\n%1"
                          LAYSTR
                        )
                      )
                    )
                    (setq ALERT_STRING
                      (acet-str-format "Layer %1 konnte nicht bereinigt werden, da er\ndurch (ein) Objekt(e) in der Zeichnung referenziert wird."
                        (car NOPURGE_LST)
                      )
                    )
                  )
                  (acet-alert ALERT_STRING)
                )
              )
            )
            (progn
              (prompt "\nOperation abgebrochen.")
              (command "_.undo" "_b")
            )
          )
        )

; ------------------------- Layer Merge --------------------------

        ((= (strcase MODE) "MERGE")

          (setq ANS  T
                LOOP T
          )

          (while LOOP
            (cond
              ((not ANS)
                (setq LOOP nil)
              )
              ((listp ANS)
                (setq NEWLAY  (cdr (assoc 8 (entget (car ANS))))
                      LOOP nil
                )
              )
              ((= ANS "Type-it")
                (setq NEWLAY (getstring "\nLayer-Namen eingeben oder [?]: " T))
                (cond
                  ((= NEWLAY "")
                    (setq ANS    nil
                          NEWLAY nil
                    )
                  )
                  ((= NEWLAY "?")
                    (setq STR (getstring "\nAufzulistende(n) Layer-Namen eingeben <*>:"))
                    (textscr)
                    (setvar "qaflags" 2)
                    (command "_.layer" "?" STR)
                    (while (< 0 (getvar "cmdactive")) (command ""))
                    (setvar "qaflags" 0)
                  )
                  ((setq NEWLAY (tblobjname "LAYER" NEWLAY))
                    (setq NEWLAY (cdr (assoc 2 (entget NEWLAY)))     ; get its real name
                          LOOP   nil
                    )
                  )
                  (T
                    (prompt "\nUngltiger Layer-Name.")
                    (setq NEWLAY nil)
                  )
                )
              )
            )

            (if (and NEWLAY (not LOOP))
              (cond
                ((member NEWLAY LAYLST)
                  (prompt "\nKann einen Layer nicht mit sich selbst mischen.")
                  (setq LOOP T)
                )
                ((wcmatch NEWLAY "*|*")
                  (prompt (acet-str-format "\nLayer %1 ist extern abhngig und kann nicht verwendet werden."  NEWLAY ))
                  (setq LOOP T)
                )
              )
            )

            (if (and LOOP (/= ANS "Type-it"))
              (progn
                (setvar "errno" 7)
                (terpri)
                (while (= (getvar "errno") 7)
                  (setvar "errno" 0)
                  (initget "Eingeben _Type-it ")
                  (setq ANS  (entsel "\rObjekt auf Ziel-Layer auswhlen oder [Eingeben]: "))
                  (if (= (getvar "errno") 7)
                    (prompt "\nNichts ausgewhlt.")
                  )
                )
              )
            )
          ); while LOOP

          (if NEWLAY
            (progn
              (terpri)
              (prompt "\n******** WARNUNG ********")
              (if BLKLST
                (progn
                  (textscr)
                  (terpri)
                  (prompt (acet-str-format "\nEs sind %1 Blockdefinition(en) vorhanden, die den (die) zu lschenden Layer referenzieren."
                            (rtos (length BLKLST) 2 0)
                           )
                  )
                  (prompt (acet-str-format "\nDer oder die Blcke werden neu definiert. Die Elemente, die\nden (die) Layer referenzieren, werden zu Referenz-Layer %1 gendert." NEWLAY))
                )
              )
              (terpri)

              (if (= (length LAYLST) 1)
                (prompt (acet-str-format "\nSie sind dabei, Layer %1 endgltig mit Layer %2 zu mischen." (car LAYLST) NEWLAY))
                (progn
                  (textscr)
                  (prompt (acet-str-format "\nSie sind dabei, die folgenden Layer endgltig mit Layer %1 zu mischen.\n"  NEWLAY ))
                  (foreach LAY LAYLST
                    (prompt (strcat "\n" LAY))
                  )
                  (terpri)
                )
              )
              (initget "Ja Nein _Yes No")
              (setq PASS (getkword "\nMchten Sie fortfahren? [Ja/Nein] <Nein>:"))
              (setq PASS (if (null PASS) "No" PASS))
              (graphscr)

              (if (and
                (= PASS "Yes")
                (setq LAYOUTLST (acet-lydelmrg-vpcheck LAYLST MODE))
              )
                (progn

                  (if BLKLST
                    (foreach BLK BLKLST
                      (acet-lydelmrg-blkredef (car BLK) (cdr BLK) MODE NEWLAY)
                    )
                  )

                  (if (member CLAY LAYLST)
                      (setvar "clayer" NEWLAY)
                  )

                  (setq CLAYOUT (getvar "ctab")
                        CVPORT (getvar "cvport")
                  )

                  (foreach LAYOUT LAYOUTLST
                    (setvar "ctab" LAYOUT)
                    
                    (if (/= LAYOUT "Model")
                      (progn
                        (command "_.pspace")
                        (command "_.vplayer" "_t" LAYSTR "_a" "" )
                      )
                    )

                    (foreach LAY LAYLST
                      (terpri)
                      (princ (acet-str-format "\nMische Layer %1 mit Layer %2." LAY NEWLAY))

                      (setq FLTR (list '(-4 . "<AND") (cons 8 LAY) (cons 410 LAYOUT) '(-4 . "AND>")))

                      (if (setq SS (ssget "_x" FLTR))
                        (command "_.chprop" SS "" "_layer" NEWLAY "")
                      )

                      (if (setq SS (ssget "_x" FLTR))
                        (progn
                          (setq CNT 0)
                          (while (and ss
                                      (> (sslength ss) 0)
                                      (setq EN (ssname SS CNT))
                                 );and
                            (setq EN (entget EN '("*"))
                                  EN (subst (cons 8 NEWLAY) (assoc 8 EN) EN)
                            )
                            (entmod EN)
                            (setq CNT (1+ CNT))
                          )
                        )
                      )
                    )
                    (setvar "ctab" CLAYOUT)

                    (if (and (/= CLAYOUT "Model")
                             (< 1 CVPORT)
                             (ssget "_x" '((0 . "VIEWPORT")))
                        )
                      (command "_.mspace")
                    )
                  )

                  (foreach LAY LAYLST
                    (acet-lydelmrg-attredef LAY MODE NEWLAY)             ; Recreate inserts which reference LAY
                    (prompt (acet-str-format "\nAlle Elemente von Layer %1 wurden auf Layer %2 verschoben." LAY NEWLAY))
                  )
                  (terpri)
                  (terpri)
                  (command "_.purge" "_layer" LAYSTR "_n")
                  (foreach LAY LAYLST
                    (if (tblobjname "LAYER" LAY)
                      (setq NOPURGE_LST (append NOPURGE_LST (list LAY)))
                    )
                  )
                  (if NOPURGE_LST
                    (progn
                      (if (< 1 (length NOPURGE_LST))
                        (progn
                          (setq LAYSTR "\n")
                          (foreach VAL NOPURGE_LST (setq LAYSTR (strcat LAYSTR VAL "\n")))
                          (setq ALERT_STRING
                            (acet-str-format "Die folgenden Layer konnten nicht bereinigt werden, da sie\ndurch (ein) Objekt(e) in der Zeichnung referenziert werden:\n%1"
                                  LAYSTR
                            )
                          )
                        )
                        (setq ALERT_STRING
                          (acet-str-format "Layer %1 konnte nicht bereinigt werden, da er\ndurch (ein) Objekt(e) in der Zeichnung referenziert wird."
                            (car NOPURGE_LST) )
                        )
                      )
                      (acet-alert ALERT_STRING)
                    )
                  )

                )

                (progn
                  (prompt "\nOperation abgebrochen.")
                  (foreach LAY LAYLST
                    (setq SS  (ssget "_x" (list (cons 8 LAY)))
                          CNT 0
                    )
                    (while (and ss
                                (> (sslength ss) 0)
                                (setq EN (ssname SS CNT))
                           );and
                      (redraw EN 4)
                      (setq CNT (1+ CNT))
                    )
                  )
                )
              )
            )
            (progn
              (prompt "\nOperation abgebrochen.")
              (foreach LAY LAYLST
                (setq SS  (ssget "_x" (list (cons 8 LAY)))
                      CNT 0
                )
                (while (and ss
                            (> (sslength ss) 0)
                            (setq EN (ssname SS CNT))
                       );and
                  (redraw EN 4)
                  (setq CNT (1+ CNT))
                )
              )
            )
          )
        )
      )
    )
    (setvar "clayer" CLAY)
  )


  (acet-error-restore)                                  ; Retsore values
  (princ)
)

